﻿using System;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Proofing;
using WindowSettings = Telerik.Windows.Documents.Proofing.WindowSettings;
using WindowStartupLocation = Telerik.Windows.Controls.WindowStartupLocation;

namespace CustomRadSpellCheckerDemo
{
    public class MySpellChecker
    {

        #region Properties

        public static WindowSettings WindowSettings { get; private set; }
        public static event EventHandler<SpellCheckingCompletedEventArgs> SpellCheckingCompleted;

        internal static void OnSpellCheckingCompleted(object sender, SpellCheckingCompletedEventArgs args)
        {
            if (MySpellChecker.SpellCheckingCompleted != null)
            {
                MySpellChecker.SpellCheckingCompleted(sender, args);
            }
        }

        #endregion

        #region Constructors

        static MySpellChecker()
        {
            MySpellChecker.WindowSettings = new WindowSettings();
            MySpellChecker.WindowSettings.StartupLocation = WindowStartupLocation.CenterScreen;
            MySpellChecker.windowSettingsForCurrentSpellCheckIteration = new WindowSettings();
        }

        #endregion

        #region Fields

        private static WindowSettings windowSettingsForCurrentSpellCheckIteration;

        #endregion

        #region Methods

        public static void Check(Control controlToCheck, SpellCheckingMode spellCheckingMode)
        {
            switch (spellCheckingMode)
            {
                case SpellCheckingMode.WordByWord:
                    MySpellChecker.CheckWordByWord(controlToCheck);
                    break;

                case SpellCheckingMode.AllAtOnce:
                    MySpellChecker.CheckAllAtOnce(controlToCheck);
                    break;

                default:
                    break;
            }
        }

        private static void CheckWordByWord(Control controlToCheck)
        {
            IControlSpellChecker spellChecker = ControlSpellCheckersManager.GetControlSpellChecker(controlToCheck.GetType());
            if (spellChecker != null)
            {
                MySpellChecker.CopyWindowSettings(MySpellChecker.WindowSettings, MySpellChecker.windowSettingsForCurrentSpellCheckIteration);

                spellChecker.CurrentControl = controlToCheck;
                MySpellChecker.ShowWindowForNextError(spellChecker);
            }
            else
            {
                throw new NotImplementedException("No spellcheck for this type of control.");
            }
        }

        private static void ShowWindowForNextError(IControlSpellChecker spellChecker)
        {
            spellChecker.ResetFields();
            IWordInfo incorrectWordInfo = spellChecker.MoveToNextError();
            if (incorrectWordInfo == null)
            {
                if (MySpellChecker.WindowSettings.ShowAlertWhenSpellCheckingCompleted)
                {
                    var parameters = new DialogParameters()
                    {
                        Header = LocalizationManager.GetString("SpellChecker_SpellingCheckIsCompleteMessageHeader"),
                        Content = LocalizationManager.GetString("SpellChecker_SpellingCheckIsCompleteMessage"),
                    };

                    if (MySpellChecker.WindowSettings.Theme != null)
                    {
                        parameters.Theme = MySpellChecker.WindowSettings.Theme;
                    }

                    //if (StyleManager.IsEnabled)
                    //{
                    //    System.Windows.Style style = new System.Windows.Style(typeof(RadWindow));
                    //    style.Setters.Add(new System.Windows.Setter(RadWindow.IsTopmostProperty, true));
                    //    parameters.WindowStyle = style;
                    //}

                    RadWindow.Alert(parameters);
                }

                MySpellChecker.OnSpellCheckingCompleted(spellChecker, new SpellCheckingCompletedEventArgs(SpellCheckingMode.WordByWord, false));

                return;
            }

            SpellCheckWordByWordWindow checkWordByWordWindow = new SpellCheckWordByWordWindow(incorrectWordInfo.Word, spellChecker);

            MySpellChecker.CopyWindowSettings(MySpellChecker.windowSettingsForCurrentSpellCheckIteration, checkWordByWordWindow);

            checkWordByWordWindow.Closed += new EventHandler<WindowClosedEventArgs>(SpellCheckWordByWordWindow_Closed);

            //NOTE: Invoke RadWindow's ShowDialog() method instead in order to show modal dialog.
            //checkWordByWordWindow.Show();
            checkWordByWordWindow.ShowDialog();

#if SILVERLIGHT
            checkWordByWordWindow.Dispatcher.BeginInvoke((Action)delegate()
            {
                spellChecker.CurrentControl.Focus();
            });
#endif
        }

        private static void CheckAllAtOnce(Control controlToCheck)
        {
            IControlSpellChecker spellChecker = ControlSpellCheckersManager.GetControlSpellChecker(controlToCheck.GetType());
            if (spellChecker != null)
            {
                spellChecker.CurrentControl = controlToCheck;
                RadDocument editingElementContentToRadDocument = spellChecker.GetContentAsDocument();
                editingElementContentToRadDocument.Measure(RadDocument.MAX_DOCUMENT_SIZE);
                SpellCheckAllAtOnceWindow checkAllAtOnceWindow = new SpellCheckAllAtOnceWindow(ControlSpellCheckersManager.GetControlSpellChecker(typeof(RadRichTextBox)));

                MySpellChecker.CopyWindowSettings(MySpellChecker.WindowSettings, checkAllAtOnceWindow);

                checkAllAtOnceWindow.CustomControl = controlToCheck;
                checkAllAtOnceWindow.Closed += SpellCheckAllAtOnceWindow_Closed;
                checkAllAtOnceWindow.ShowDialog(editingElementContentToRadDocument, true);
            }
            else
            {
                throw new NotImplementedException("No spellcheck for this type of control. If you are using Library Caching " +
                    "please refer to our online documentation for more information on how to register the ControlSpellCheckers manually.");
            }
        }

        #endregion


        #region Event Handlers

        private static void SpellCheckWordByWordWindow_Closed(object sender, WindowClosedEventArgs e)
        {
            SpellCheckWordByWordWindow spellCheckingWindow = sender as SpellCheckWordByWordWindow;

            MySpellChecker.CopyWindowSettings(spellCheckingWindow, MySpellChecker.windowSettingsForCurrentSpellCheckIteration);

            if (spellCheckingWindow != null)
            {
                if (spellCheckingWindow.DialogResult == null ||
                    (spellCheckingWindow.DialogResult.HasValue && spellCheckingWindow.DialogResult.Value == false))
                {
                    MySpellChecker.OnSpellCheckingCompleted(sender, new SpellCheckingCompletedEventArgs(SpellCheckingMode.WordByWord, true));
                    return;
                }

                MySpellChecker.ShowWindowForNextError(spellCheckingWindow.SpellCheckingManager);
            }
        }

        private static void SpellCheckAllAtOnceWindow_Closed(object sender, WindowClosedEventArgs e)
        {
            SpellCheckAllAtOnceWindow spellCheckAllAtOnceWindow = (SpellCheckAllAtOnceWindow)sender;
            if (spellCheckAllAtOnceWindow.DialogResult.HasValue && spellCheckAllAtOnceWindow.DialogResult.Value == true)
            {
                IControlSpellChecker spellChecker = ControlSpellCheckersManager.GetControlSpellChecker(spellCheckAllAtOnceWindow.CustomControl.GetType());
                spellChecker.CurrentControl = spellCheckAllAtOnceWindow.CustomControl;
                spellChecker.SetContentFromDocument(spellCheckAllAtOnceWindow.SpellCheckedDocument);

                MySpellChecker.OnSpellCheckingCompleted(spellChecker, new SpellCheckingCompletedEventArgs(SpellCheckingMode.AllAtOnce, false));
                return;
            }

            MySpellChecker.OnSpellCheckingCompleted(sender, new SpellCheckingCompletedEventArgs(SpellCheckingMode.AllAtOnce, true));
        }

        private static void CopyWindowSettings(WindowSettings from, WindowSettings to)
        {
            to.Top = from.Top;
            to.TopOffset = from.TopOffset;
            to.Left = from.Left;
            to.LeftOffset = from.LeftOffset;
            to.StartupLocation = from.StartupLocation;
            to.Theme = from.Theme;
        }

        private static void CopyWindowSettings(RadWindow from, WindowSettings to)
        {
            to.Top = from.Top;
            to.Left = from.Left;

            if (from is SpellCheckAllAtOnceWindow)
            {
                to.Theme = ((SpellCheckAllAtOnceWindow)from).SelectedTheme;
            }

            if (from is SpellCheckWordByWordWindow)
            {
                to.Theme = ((SpellCheckWordByWordWindow)from).SelectedTheme;
            }

            if ((from.Top != 0 || from.Left != 0) && (from.WindowStartupLocation == WindowStartupLocation.CenterScreen || from.WindowStartupLocation == WindowStartupLocation.CenterOwner))
            {
                to.StartupLocation = WindowStartupLocation.Manual;
            }
            else
            {
                to.StartupLocation = from.WindowStartupLocation;
            }
        }

        private static void CopyWindowSettings(WindowSettings from, RadWindow to)
        {
            to.Top = from.Top;
            to.Left = from.Left;
            to.WindowStartupLocation = from.StartupLocation;

            if (to is SpellCheckAllAtOnceWindow)
            {
                ((SpellCheckAllAtOnceWindow)to).SelectedTheme = from.Theme;
            }

            if (to is SpellCheckWordByWordWindow)
            {
                ((SpellCheckWordByWordWindow)to).SelectedTheme = from.Theme;
            }
        }

        #endregion
    }
}
